/*
 * Copyright (C) 1997-2011 Andre Noll <maan@systemlinux.org>
 *
 * Licensed under the GPL v2. For licencing details see COPYING.
 */

/** \file client.c the client program used to connect to para_server */

#include <regex.h>
#include <stdbool.h>

#include "para.h"
#include "list.h"
#include "sched.h"
#include "crypt.h"
#include "client.cmdline.h"
#include "string.h"
#include "stdin.h"
#include "stdout.h"
#include "client.h"
#include "buffer_tree.h"
#include "error.h"

INIT_CLIENT_ERRLISTS;

static struct client_task *ct;
static struct stdin_task sit;
static struct stdout_task sot;

static void supervisor_post_select(__a_unused struct sched *s, struct task *t)
{
	if (ct->task.error < 0) {
		t->error = ct->task.error;
		return;
	}
	if (ct->status == CL_SENDING) {
		stdin_set_defaults(&sit);
		register_task(&sit.task);
		t->error = -E_TASK_STARTED;
		return;
	}
	if (ct->status == CL_RECEIVING) {
		stdout_set_defaults(&sot);
		register_task(&sot.task);
		t->error = -E_TASK_STARTED;
		return;
	}
}

static struct task svt = {
	.post_select = supervisor_post_select,
	.status = "supervisor task"
};

static int client_loglevel = LL_ERROR; /* loglevel */
INIT_STDERR_LOGGING(client_loglevel);

/**
 * The client program to connect to para_server.
 *
 * \param argc Usual argument count.
 * \param argv Usual argument vector.
 *
 * It registers two tasks: The client task that communicates with para_server
 * and the supervisor task that minitors whether the client task intends to
 * read from stdin or write to stdout.
 *
 * Once it has been determined whether the client command corresponds to a
 * stdin command (addmood, addimg, ..), either the stdin task or the stdout
 * task is set up to replace the supervisor task.
 *
 * \return EXIT_SUCCESS or EXIT_FAILURE
 *
 * \sa client_open(), stdin.c, stdout.c, para_client(1), para_server(1)
 */
int main(int argc, char *argv[])
{

	int ret;
	static struct sched s;

	init_random_seed_or_die();
	s.default_timeout.tv_sec = 1;
	s.default_timeout.tv_usec = 0;
	/*
	 * We add buffer tree nodes for stdin and stdout even though
	 * only one of them will be needed. This simplifies the code
	 * a bit wrt. to the buffer tree setup.
	 */
	sit.btrn = btr_new_node(&(struct btr_node_description)
		EMBRACE(.name = "stdin"));
	ret = client_open(argc, argv, &ct, &client_loglevel, sit.btrn, NULL);
	if (ret < 0)
		goto out;
	sot.btrn = btr_new_node(&(struct btr_node_description)
		EMBRACE(.name = "stdout", .parent = ct->btrn));
	register_task(&svt);
	ret = schedule(&s);
out:
	client_close(ct);
	btr_free_node(sit.btrn);
	btr_free_node(sot.btrn);
	if (ret < 0) {
		/* can not use PARA_LOG here because ct is NULL */
		fprintf(stderr, "%s\n", para_strerror(-ret));
		return EXIT_FAILURE;
	}
	return EXIT_SUCCESS;
}
